@@ -1,7 +1,7 @@
-#' @importFrom utils head object.size str tail
-#' @importFrom stats aggregate na.omit rbinom runif setNames
+#' @importFrom utils head object.size str tail packageVersion compareVersion
+#' @importFrom stats aggregate dist na.omit rbinom runif setNames
 #' @importFrom tools file_ext file_path_sans_ext
-#' @importFrom methods as new slot slotNames
+#' @importFrom methods as new slot slotNames "slot<-"
 #' @importFrom grid convertHeight convertUnit convertWidth current.viewport linesGrob nullGrob pathGrob pointsGrob polylineGrob unit viewport
 #' @import graphics
 #' @importFrom grDevices dev.size rgb
@@ -9,7 +9,7 @@
 #' @importFrom DBI dbConnect dbDisconnect dbExecute dbGetQuery dbReadTable dbSendQuery dbWriteTable
 #' @importFrom units as_units drop_units make_unit_label set_units
 #' @importFrom classInt classIntervals
-#' @useDynLib sf
+#' @useDynLib sf, .registration=TRUE
 NULL
 
 #' @importFrom magrittr %>%
@@ -24,6 +24,7 @@ setOldClass(c("sfc_MULTILINESTRING", "sfc"))
 setOldClass(c("sfc_POLYGON", "sfc"))
 setOldClass(c("sfc_MULTIPOLYGON", "sfc"))
 setOldClass(c("sfc_GEOMETRY", "sfc"))
+setOldClass(c("sfc_GEOMETRYCOLLECTION", "sfc"))
 setOldClass("sfg")
 setOldClass("crs")
 
@@ -37,7 +38,8 @@ pathGrob <- NULL
 		}
 	} # nocov end
 	load_gdal() 
-	assign(".sf.use_s2", Sys.getenv("_SF_USE_S2") == "true", envir=.sf_cache)
+	if ((s2 <- Sys.getenv("_SF_USE_S2")) != "")
+		options(sf_use_s2 = s2 != "false")
 }
 
 .onUnload = function(libname, pkgname) {
@@ -46,8 +48,10 @@ pathGrob <- NULL
 
 .onAttach = function(libname, pkgname) {
 	m = paste0("Linking to GEOS ", strsplit(CPL_geos_version(TRUE), "-")[[1]][1],
-		", GDAL ", CPL_gdal_version(), ", PROJ ", CPL_proj_version())
-	packageStartupMessage(m)
+		", GDAL ", CPL_gdal_version(), ", PROJ ", CPL_proj_version(),
+		"; sf_use_s2() is ", sf_use_s2())
+	m = strwrap(m, width = getOption("width"))
+	packageStartupMessage(paste0(m, collapse = "\n"))
 	if (length(grep(CPL_geos_version(FALSE, TRUE), CPL_geos_version(TRUE))) != 1) { # nocov start
 		packageStartupMessage("WARNING: different compile-time and runtime versions for GEOS found:")
 		packageStartupMessage(paste(
@@ -62,10 +66,10 @@ pathGrob <- NULL
 #' Provide the external dependencies versions of the libraries linked to sf
 #' @export
 sf_extSoftVersion = function() {
-	structure(c(CPL_geos_version(), CPL_gdal_version(), CPL_proj_version(),
+	structure(c(CPL_geos_version(), CPL_gdal_version(), CPL_proj_version(), 
 		ifelse(CPL_gdal_with_geos(), "true", "false"),
-		ifelse(CPL_proj_h(), "true", "false")),
-		names = c("GEOS", "GDAL", "proj.4", "GDAL_with_GEOS", "USE_PROJ_H"))
+		ifelse(CPL_proj_h(), "true", "false"), CPL_proj_version()),
+		names = c("GEOS", "GDAL", "proj.4", "GDAL_with_GEOS", "USE_PROJ_H", "PROJ"))
 }
 
 load_gdal <- function() {
@@ -107,13 +111,15 @@ unload_gdal <- function() {
 #' @export
 #' @name sf_project
 #' @details \code{sf_add_proj_units} loads the PROJ units `link`, `us_in`, `ind_yd`, `ind_ft`, and `ind_ch` into the udunits database, and returns \code{TRUE} invisibly on success.
+#' @examples
+#' sf_add_proj_units()
 sf_add_proj_units = function() {
 	#nocov start
-	units::install_conversion_constant("m", "link", 0.201168)
-	units::install_conversion_constant("m", "us_in", 1./39.37)
-	units::install_conversion_constant("m", "ind_yd", 0.91439523)
-	units::install_conversion_constant("m", "ind_ft", 0.30479841)
-	units::install_conversion_constant("m", "ind_ch", 20.11669506)
+	units::install_unit("link", "0.201168 m")
+	units::install_unit("us_in", "1./39.37 m")
+	units::install_unit("ind_yd", "0.91439523 m")
+	units::install_unit("ind_ft", "0.30479841 m")
+	units::install_unit("ind_ch", "20.11669506 m")
 	invisible(TRUE)
 	#nocov end
 }
