@@ -22,6 +22,8 @@ std::vector<OGRFieldType> SetupFields(OGRLayer *poLayer, Rcpp::List obj, bool up
 			ret[i] = OFTDate;
 		else if (strcmp(cls[i], "POSIXct") == 0)
 			ret[i] = OFTDateTime;
+		else if (strcmp(cls[i], "list") == 0) // list with raw vectors; #1721
+			ret[i] = OFTBinary;
 		else { // #nocov start
 			Rcpp::Rcout << "Field " << nm[i] << " of type " << cls[i] << " not supported." << std::endl;
 			Rcpp::stop("Layer creation failed.\n");
@@ -49,14 +51,14 @@ Rcpp::NumericVector get_dbl6(Rcpp::List in) {
 }
 
 void SetNull(OGRFeature *poFeature, size_t field) {
-#if (GDAL_VERSION_MINOR >= 2 || GDAL_VERSION_MAJOR > 2)
+#if GDAL_VERSION_NUM >= 2020000
 	poFeature->SetFieldNull(field);
 #else
 	poFeature->UnsetField(field);
 #endif
 }
 
-void SetFields(OGRFeature *poFeature, std::vector<OGRFieldType> tp, Rcpp::List obj, size_t i, bool shape) {
+void SetFields(OGRFeature *poFeature, std::vector<OGRFieldType> tp, Rcpp::List obj, size_t i) {
 	Rcpp::CharacterVector nm  = obj.attr("names");
 	for (size_t j = 0; j < tp.size(); j++) {
 		if (i == 0 && poFeature->GetFieldIndex(nm[j]) == -1) {
@@ -70,34 +72,35 @@ void SetFields(OGRFeature *poFeature, std::vector<OGRFieldType> tp, Rcpp::List o
 			case OFTString: {
 				Rcpp::CharacterVector cv;
 				cv = obj[j];
-				if (! Rcpp::CharacterVector::is_na(cv[i])) {
-					if (shape)
-						poFeature->SetField(j, (const char *) cv[i]);
-					else
-						poFeature->SetField(nm[j], (const char *) cv[i]);
-				} else
+				if (! Rcpp::CharacterVector::is_na(cv[i]))
+					poFeature->SetField(j, (const char *) cv[i]);
+				else
 					SetNull(poFeature, j);
 				} break;
 			case OFTInteger: {
-				Rcpp::IntegerVector iv;
-				iv = obj[j];
-				if (! Rcpp::IntegerVector::is_na(iv[i])) {
-					if (shape)
+				const OGRFieldDefn *def = poFeature->GetFieldDefnRef(j);
+				if (def->GetSubType() == OFSTBoolean) {
+					Rcpp::LogicalVector lv;
+					lv = obj[j];
+					if (! Rcpp::LogicalVector::is_na(lv[i]))
+						poFeature->SetField(j, (int) lv[i]);
+					else
+						SetNull(poFeature, j); // #nocov
+				} else { // integer:
+					Rcpp::IntegerVector iv;
+					iv = obj[j];
+					if (! Rcpp::IntegerVector::is_na(iv[i]))
 						poFeature->SetField(j, (int) iv[i]);
 					else
-						poFeature->SetField(nm[j], (int) iv[i]);
-				} else
-					SetNull(poFeature, j); // #nocov
+						SetNull(poFeature, j); // #nocov
+				}
 				} break;
 			case OFTReal: {
 				Rcpp::NumericVector nv;
 				nv = obj[j];
-				if (! Rcpp::NumericVector::is_na(nv[i])) {
-					if (shape)
-						poFeature->SetField(j, (double) nv[i]);
-					else
-						poFeature->SetField(nm[j], (double) nv[i]);
-				} else
+				if (! Rcpp::NumericVector::is_na(nv[i]))
+					poFeature->SetField(j, (double) nv[i]);
+				else
 					SetNull(poFeature, j);
 				} break;
 			case OFTDate: {
@@ -111,12 +114,8 @@ void SetFields(OGRFeature *poFeature, std::vector<OGRFieldType> tp, Rcpp::List o
 				nv0[0] = nv[i];
 				nv0.attr("class") = "Date";
 				Rcpp::Function as_POSIXlt_Date("as.POSIXlt.Date");
-				Rcpp::Function unlist("unlist");
-				Rcpp::NumericVector ret = unlist(as_POSIXlt_Date(nv0)); // use R
-				if (shape)
-					poFeature->SetField(j, 1900 + (int) ret[5], (int) ret[4] + 1, (int) ret[3]);
-				else
-					poFeature->SetField(nm[j], 1900 + (int) ret[5], (int) ret[4] + 1, (int) ret[3]);
+				Rcpp::NumericVector ret = get_dbl6(as_POSIXlt_Date(nv0));
+				poFeature->SetField(j, 1900 + (int) ret[5], (int) ret[4] + 1, (int) ret[3]);
 				} break;
 			case OFTDateTime: {
 				Rcpp::NumericVector nv;
@@ -130,15 +129,26 @@ void SetFields(OGRFeature *poFeature, std::vector<OGRFieldType> tp, Rcpp::List o
 				nv0.attr("tzone") = "UTC";
 				Rcpp::Function as_POSIXlt_POSIXct("as.POSIXlt.POSIXct");
 				Rcpp::NumericVector rd = get_dbl6(as_POSIXlt_POSIXct(nv0)); // use R
-				if (shape)
-					poFeature->SetField(j, 1900 + (int) rd[5], (int) rd[4] + 1, // #nocov start
+				poFeature->SetField(j, 1900 + (int) rd[5], (int) rd[4] + 1, // #nocov start
 						(int) rd[3], (int) rd[2], (int) rd[1],
 						(float) rd[0], 100); // nTZFlag: 0=unkown, 1=local, 100=GMT; #nocov end
-				else
-					poFeature->SetField(nm[j], 1900 + (int) rd[5], (int) rd[4] + 1,
-						(int) rd[3], (int) rd[2], (int) rd[1],
-						(float) rd[0], 100); // nTZFlag 0: unkown; 1: local; 100: GMT
 				} break;
+			case OFTBinary: 
+#if GDAL_VERSION_NUM > 3000000
+				{
+				Rcpp::List lv;
+				lv = obj[j];
+				Rcpp::RawVector rv;
+				rv = lv(0);
+				if (rv.size() == 0)
+					SetNull(poFeature, j); // #nocov
+				else {
+					const void *ptr = &(rv[0]);
+					int size = rv.size();
+					poFeature->SetField(j, size, ptr);
+				}
+				} break;
+#endif
 			default:
 				// we should never get here! // #nocov start
 				Rcpp::Rcout << "field with unsupported type ignored" << std::endl; 
@@ -152,10 +162,12 @@ void SetFields(OGRFeature *poFeature, std::vector<OGRFieldType> tp, Rcpp::List o
 int CPL_write_ogr(Rcpp::List obj, Rcpp::CharacterVector dsn, Rcpp::CharacterVector layer,
 	Rcpp::CharacterVector driver, Rcpp::CharacterVector dco, Rcpp::CharacterVector lco,
 	Rcpp::List geom, Rcpp::CharacterVector dim, Rcpp::CharacterVector fids,
+	Rcpp::CharacterVector ConfigOptions,
 	bool quiet, Rcpp::LogicalVector append, bool delete_dsn = false, bool delete_layer = false,
-	bool write_geometries = true) {
+	bool write_geometries = true, int width = 80) {
 
 	// init:
+	set_config_options(ConfigOptions);
 	if (driver.size() != 1 || dsn.size() != 1 || layer.size() != 1)
 		Rcpp::stop("argument dsn, layer or driver not of length 1.\n");
 
@@ -250,9 +262,12 @@ int CPL_write_ogr(Rcpp::List obj, Rcpp::CharacterVector dsn, Rcpp::CharacterVect
 				Rcpp::stop("Creation failed.\n");
 			}
 		}
-		if (! quiet)
-			Rcpp::Rcout << "Writing layer `" << layer[0] << "' to data source `" << dsn[0] <<
-				"' using driver `" << driver[0] << "'" << std::endl;
+		if (! quiet) {
+			Rcpp::Rcout << "Writing layer `" << layer[0] << "' to data source ";
+			if (LENGTH(dsn[0]) > width - (44 + LENGTH(layer[0]) + LENGTH(driver[0])))
+				Rcpp::Rcout << std::endl << "  ";
+			Rcpp::Rcout << "`" << dsn[0] << "' using driver `" << driver[0] << "'" << std::endl;
+		}
 	}
 
 	// can & do transaction?
@@ -315,7 +330,7 @@ int CPL_write_ogr(Rcpp::List obj, Rcpp::CharacterVector dsn, Rcpp::CharacterVect
 
 	for (size_t i = 0; i < geomv.size(); i++) { // create all features & add to layer:
 		OGRFeature *poFeature = OGRFeature::CreateFeature(poLayer->GetLayerDefn());
-		SetFields(poFeature, fieldTypes, obj, i, driver[0] == "ESRI Shapefile");
+		SetFields(poFeature, fieldTypes, obj, i);
 		if (write_geometries)
 			poFeature->SetGeometryDirectly(geomv[i]);
 		if (fids.size() > (int) i)
@@ -332,9 +347,10 @@ int CPL_write_ogr(Rcpp::List obj, Rcpp::CharacterVector dsn, Rcpp::CharacterVect
 			        Rcpp::Rcout << "Deleting layer `" << layer[0] << "' failed" << std::endl;
 			} // #nocov end
 			OGRFeature::DestroyFeature(poFeature);
-			if (transaction)
+			if (transaction) {
+				unset_config_options(ConfigOptions);
 				return 1; // try once more, writing to tmp file and copy #nocov
-			else
+			} else
 				Rcpp::stop("Feature creation failed.\n");
 		}
 		OGRFeature::DestroyFeature(poFeature); // deletes geom[i] as well
@@ -345,5 +361,83 @@ int CPL_write_ogr(Rcpp::List obj, Rcpp::CharacterVector dsn, Rcpp::CharacterVect
 		Rcpp::stop("CommitTransaction() failed.\n"); 
 	} // #nocov end
 	GDALClose(poDS);
+	unset_config_options(ConfigOptions);
 	return 0; // all O.K.
 }
+
+// delete a data source, or one or more layers within a data source
+// [[Rcpp::export]]
+int CPL_delete_ogr(Rcpp::CharacterVector dsn, Rcpp::CharacterVector layer,
+	Rcpp::CharacterVector driver, bool quiet = true) {
+
+	// init:
+	if (driver.size() != 1 || dsn.size() != 1)
+		Rcpp::stop("argument dsn or driver not of length 1.\n");
+
+	/* GDALAllRegister(); -- has been done during .onLoad() */
+	// get driver:
+	GDALDriver *poDriver = GetGDALDriverManager()->GetDriverByName(driver[0]);
+	if (poDriver == NULL) {
+		Rcpp::Rcout << "driver `" << driver[0] << "' not available." << std::endl;
+		Rcpp::stop("Driver not available.\n");
+	}
+
+	// delete data source:
+	if (layer.size() == 0) {
+		if (poDriver->Delete(dsn[0]) != CE_None)
+			Rcpp::Rcout << "Deleting source `" << dsn[0] << "' failed" << std::endl;
+		else if (! quiet)
+			Rcpp::Rcout << "Deleting source `" << dsn[0] << "' using driver `" << driver[0] << "'" << std::endl;
+		return 0;
+	} 
+
+	// delete layer(s):
+
+	// data set:
+	GDALDataset *poDS = (GDALDataset *) GDALOpenEx(dsn[0], GDAL_OF_VECTOR | GDAL_OF_UPDATE, NULL, NULL, NULL); 
+	if (poDS == NULL) {
+		Rcpp::Rcout << "Data source `" << dsn[0] << "' not found" << std::endl;
+		return 1;
+	}
+
+	bool can_do_transaction = (poDS->TestCapability(ODsCTransactions) == TRUE); // can?
+	bool transaction = false;
+	if (can_do_transaction) { // try to start transaction:
+		unset_error_handler();
+		transaction = (poDS->StartTransaction() == OGRERR_NONE); // do?
+		set_error_handler();
+		if (! transaction) { // failed: #nocov start
+			GDALClose(poDS);
+			Rcpp::Rcout << "On data source `" << dsn[0] << "' cannot start transaction" << std::endl;
+			return 1; // transaction failed!
+		} // #nocov end
+	}
+
+	for (int i = 0; i < layer.size(); i++) { // reverse loop order if inefficient?
+		// find & delete layer:
+		for (int iLayer = 0; iLayer < poDS->GetLayerCount(); iLayer++) {
+			OGRLayer *poLayer = poDS->GetLayer(iLayer);
+			if (poLayer != NULL && EQUAL(poLayer->GetName(), layer[i])) {
+				OGRErr err = poDS->DeleteLayer(iLayer);
+				if (! quiet) {
+					if (err == OGRERR_UNSUPPORTED_OPERATION)
+						Rcpp::Rcout << "Deleting layer not supported by driver `" << driver[0] << "'"  // #nocov
+							<< std::endl; // #nocov
+					else  {
+						Rcpp::Rcout << "Deleting layer `" << layer[0] << "' using driver `" << 
+							driver[0] << "'" << std::endl;
+					}
+				}
+				if (err != OGRERR_NONE)
+					Rcpp::Rcout << "Deleting layer `" << layer[i] << "' failed" << std::endl;
+			}
+		}
+	}
+	if (transaction && poDS->CommitTransaction() != OGRERR_NONE) { // #nocov start
+		poDS->RollbackTransaction();
+		Rcpp::Rcout << "CommitTransaction() failed." << std::endl; 
+		return 1;
+	} // #nocov end
+	GDALClose(poDS);
+	return 0;
+}
