@@ -29,8 +29,12 @@
 #  endif
 # endif
 # if GEOS_VERSION_MINOR >= 10
+#  define HAVE310
 #  define HAVE3101
 # endif
+# if GEOS_VERSION_MINOR >= 11
+#  define HAVE311
+# endif
 #else
 # if GEOS_VERSION_MAJOR > 3
 #  define HAVE340
@@ -39,7 +43,9 @@
 #  define HAVE361
 #  define HAVE380
 #  define HAVE390
+#  define HAVE310
 #  define HAVE3101
+#  define HAVE311
 # endif
 #endif
 
@@ -56,6 +62,7 @@ typedef char (* log_fn)(GEOSContextHandle_t, const GEOSGeometry *, const GEOSGeo
 typedef char (* log_prfn)(GEOSContextHandle_t, const GEOSPreparedGeometry *,
 	const GEOSGeometry *);
 typedef GEOSGeom (* geom_fn)(GEOSContextHandle_t, const GEOSGeom, const GEOSGeom);
+typedef GEOSGeom (* geom_fnp)(GEOSContextHandle_t, const GEOSGeom, const GEOSGeom, double grid_size);
 
 static int notice = 0; // global var to silently catch notice of illegal geoms, e.g. non-closed rings
 
@@ -103,6 +110,18 @@ static void __countErrorHandler(const char *fmt, void *userdata) {
 }
 
 static void __emptyNoticeHandler(const char *fmt, void *userdata) { }
+
+static void __checkInterruptFn(void*) {
+	R_CheckUserInterrupt();
+}
+
+
+static void __checkInterrupt() {
+	// Adapted from Rcpp/Interrupt.h
+	if (!R_ToplevelExec(__checkInterruptFn, nullptr)) {
+		GEOS_interruptRequest();
+	}
+}
 // #nocov end
 
 GEOSContextHandle_t CPL_geos_init(void) {
@@ -110,6 +129,8 @@ GEOSContextHandle_t CPL_geos_init(void) {
 	GEOSContextHandle_t ctxt = GEOS_init_r();
 	GEOSContext_setNoticeHandler_r(ctxt, __warningHandler);
 	GEOSContext_setErrorHandler_r(ctxt, __errorHandler);
+	GEOS_interruptRegisterCallback(__checkInterrupt);
+
 	return ctxt;
 #else
 	return initGEOS_r((GEOSMessageHandler) __warningHandler, (GEOSMessageHandler) __errorHandler);
@@ -150,6 +171,17 @@ static std::vector<GEOSGeometry*> to_raw(std::vector<GeomPtr> & g) {
 	return raw;
 }
 
+double geos_grid_size(Rcpp::List x) {
+	double precision = x.attr("precision");
+	if (precision != 0.0)
+		precision = 1. / precision;
+	return precision;
+}
+
+double geos_grid_size_xy(Rcpp::List x, Rcpp::List y) {
+	return std::max(geos_grid_size(x), geos_grid_size(y));
+}
+
 std::vector<GeomPtr> geometries_from_sfc(GEOSContextHandle_t hGEOSCtxt, Rcpp::List sfc, int *dim = NULL, bool stop_on_NULL = true) {
 
 	Rcpp::List sfc_cls = get_dim_sfc(sfc);
@@ -165,10 +197,8 @@ std::vector<GeomPtr> geometries_from_sfc(GEOSContextHandle_t hGEOSCtxt, Rcpp::Li
 		Rcpp::stop("GEOS does not support XYM or XYZM geometries; use st_zm() to drop M\n"); // #nocov
 
 #ifdef HAVE_390
-	double precision = sfc.attr("precision");
-	bool set_precision = precision != 0.0;
-	if (set_precision)
-		precision = 1/precision;
+	double grid_size = geos_grid_size(sfc);
+	bool set_precision = grid_size != 0.0;
 	sfc.attr("precision") = 0.0; // so that CPL_write_wkb doesn't do the rounding;
 #endif
 	Rcpp::List wkblst = CPL_write_wkb(sfc, true);
@@ -185,7 +215,7 @@ std::vector<GeomPtr> geometries_from_sfc(GEOSContextHandle_t hGEOSCtxt, Rcpp::Li
 		}
 #ifdef HAVE_390
 		else if (set_precision)
-			g[i] = geos_ptr(GEOSGeom_setPrecision_r(hGEOSCtxt, g[i].get(), precision, 0), hGEOSCtxt);
+			g[i] = geos_ptr(GEOSGeom_setPrecision_r(hGEOSCtxt, g[i].get(), grid_size, GEOS_PREC_VALID_OUTPUT), hGEOSCtxt);
 #endif
 	}
 	GEOSWKBReader_destroy_r(hGEOSCtxt, wkb_reader);
@@ -267,7 +297,8 @@ log_fn which_geom_fn(const std::string op) {
 		return GEOSCovers_r;
 	else if (op == "covered_by")
 		return GEOSCoveredBy_r;
-	Rcpp::stop("wrong value for op"); // unlikely to happen unless user wants to #nocov
+	Rcpp::stop("wrong value for op: please report as issue"); // unlikely to happen unless user wants to #nocov
+	return GEOSCoveredBy_r; // never reached; satisfy -Wreturn-type #nocov
 }
 
 log_prfn which_prep_geom_fn(const std::string op) {
@@ -294,6 +325,7 @@ log_prfn which_prep_geom_fn(const std::string op) {
 	else if (op == "covered_by")
 		return GEOSPreparedCoveredBy_r;
 	Rcpp::stop("wrong value for op"); // unlikely to happen unless user wants to #nocov
+	return GEOSPreparedCoveredBy_r; // never reached; satisfy -Wreturn-type #nocov
 }
 
 /*
@@ -781,6 +813,15 @@ Rcpp::List CPL_geos_op(std::string op, Rcpp::List sfc,
 	} else if (op == "boundary") {
 		for (size_t i = 0; i < g.size(); i++)
 			out[i] = geos_ptr(chkNULL(GEOSBoundary_r(hGEOSCtxt, g[i].get())), hGEOSCtxt);
+	} else if (op == "concave_hull") {
+#ifdef HAVE311
+		double ratio = bufferDist[0];
+		unsigned int allowHoles = preserveTopology[0];
+		for (size_t i = 0; i < g.size(); i++)
+			out[i] = geos_ptr(chkNULL(GEOSConcaveHull_r(hGEOSCtxt, g[i].get(), ratio, allowHoles)), hGEOSCtxt);
+#else
+		Rcpp::stop("st_concave_hull requires GEOS >= 3.11");
+#endif
 	} else if (op == "convex_hull") {
 		for (size_t i = 0; i < g.size(); i++)
 			out[i] = geos_ptr(chkNULL(GEOSConvexHull_r(hGEOSCtxt, g[i].get())), hGEOSCtxt);
@@ -820,6 +861,13 @@ Rcpp::List CPL_geos_op(std::string op, Rcpp::List sfc,
 				dTolerance[i], bOnlyEdges)), hGEOSCtxt);
 	} else
 #endif
+#ifdef HAVE310
+	if (op == "triangulate_constrained") {
+		for (size_t i = 0; i < g.size(); i++)
+			out[i] = geos_ptr(chkNULL(GEOSConstrainedDelaunayTriangulation_r(hGEOSCtxt, g[i].get())), 
+							hGEOSCtxt);
+	} else
+#endif
 #ifdef HAVE370
 	if (op == "reverse") {
 		for (size_t i = 0; i < g.size(); i++) {
@@ -895,6 +943,9 @@ Rcpp::List CPL_geos_op2(std::string op, Rcpp::List sfcx, Rcpp::List sfcy) {
 	std::vector<GeomPtr> out;
 	std::vector<double> index_x, index_y;
 	std::vector<size_t> items(x.size());
+#ifdef HAVE_390
+	double grid_size = geos_grid_size_xy(sfcx, sfcy);
+#endif
 
 	if (op == "intersection") {
 
@@ -917,7 +968,11 @@ Rcpp::List CPL_geos_op2(std::string op, Rcpp::List sfcx, Rcpp::List sfcy) {
 			std::sort(sel.begin(), sel.end());
 			for (size_t item = 0; item < sel.size(); item++) {
 				size_t j = sel[item];
+#ifndef HAVE_390
 				GeomPtr geom = geos_ptr(GEOSIntersection_r(hGEOSCtxt, x[j].get(), y[i].get()), hGEOSCtxt);
+#else
+				GeomPtr geom = geos_ptr(GEOSIntersectionPrec_r(hGEOSCtxt, x[j].get(), y[i].get(), grid_size), hGEOSCtxt);
+#endif
 				if (geom == nullptr)
 					Rcpp::stop("GEOS exception"); // #nocov
 				if (! chk_(GEOSisEmpty_r(hGEOSCtxt, geom.get()))) {
@@ -930,6 +985,7 @@ Rcpp::List CPL_geos_op2(std::string op, Rcpp::List sfcx, Rcpp::List sfcy) {
 		}
 
 	} else {
+#ifndef HAVE_390
 		geom_fn geom_function;
 		if (op == "union")
 			geom_function = (geom_fn) GEOSUnion_r;
@@ -937,12 +993,25 @@ Rcpp::List CPL_geos_op2(std::string op, Rcpp::List sfcx, Rcpp::List sfcy) {
 			geom_function = (geom_fn) GEOSDifference_r;
 		else if (op == "sym_difference")
 			geom_function = (geom_fn) GEOSSymDifference_r;
+#else
+		geom_fnp geom_function;
+		if (op == "union")
+			geom_function = (geom_fn) GEOSUnionPrec_r;
+		else if (op == "difference")
+			geom_function = (geom_fn) GEOSDifferencePrec_r;
+		else if (op == "sym_difference")
+			geom_function = (geom_fn) GEOSSymDifferencePrec_r;
+#endif
 		else
 			Rcpp::stop("invalid operation"); // #nocov
 
 		for (size_t i = 0; i < y.size(); i++) {
 			for (size_t j = 0; j < x.size(); j++) {
+#ifndef HAVE_390
 				GeomPtr geom = geos_ptr(geom_function(hGEOSCtxt, x[j].get(), y[i].get()), hGEOSCtxt);
+#else
+				GeomPtr geom = geos_ptr(geom_function(hGEOSCtxt, x[j].get(), y[i].get(), grid_size), hGEOSCtxt);
+#endif
 				if (geom == nullptr)
 					Rcpp::stop("GEOS exception"); // #nocov
 				if (! chk_(GEOSisEmpty_r(hGEOSCtxt, geom.get()))) {
@@ -1066,10 +1135,12 @@ Rcpp::List CPL_geos_nearest_points(Rcpp::List sfc0, Rcpp::List sfc1, bool pairwi
 		out = sfc_from_geometry(hGEOSCtxt, ls, dim);
 	} else {
 		std::vector<GeomPtr> ls(sfc0.size() * sfc1.size());
-		for (size_t i = 0; i < gmv0.size(); i++)
+		for (size_t i = 0; i < gmv0.size(); i++) {
 			for (size_t j = 0; j < gmv1.size(); j++)
 				ls[(i * gmv1.size()) + j] =
 					geos_ptr(GEOSGeom_createLineString_r(hGEOSCtxt, GEOSNearestPoints_r(hGEOSCtxt, gmv0[i].get(), gmv1[j].get())), hGEOSCtxt); // converts as LINESTRING
+			R_CheckUserInterrupt();
+		}
 		out = sfc_from_geometry(hGEOSCtxt, ls, dim);
 	}
 
@@ -1116,6 +1187,9 @@ Rcpp::List CPL_nary_difference(Rcpp::List sfc) {
 	GEOSContextHandle_t hGEOSCtxt = CPL_geos_init();
 	std::vector<GeomPtr> x = geometries_from_sfc(hGEOSCtxt, sfc, &dim);
 	std::vector<GeomPtr> out;
+#ifdef HAVE_390
+	double grid_size = geos_grid_size(sfc);
+#endif
 	// initialize trees to find overlapping areas quickly
 	for (size_t i = 0; i < x.size(); i++) {
 		// if i'th geometry in x is empty then skip it
@@ -1145,7 +1219,11 @@ Rcpp::List CPL_nary_difference(Rcpp::List sfc) {
 					// test if the items intersect with geom
 					if (chk_(GEOSIntersects_r(hGEOSCtxt, geom.get(), out[tree_sel[j]].get()))) {
 						// if they do then erase overlapping parts from geom
+#ifndef HAVE_390
 						geom = geos_ptr(GEOSDifference_r(hGEOSCtxt, geom.get(), out[tree_sel[j]].get()), hGEOSCtxt);
+#else
+						geom = geos_ptr(GEOSDifferencePrec_r(hGEOSCtxt, geom.get(), out[tree_sel[j]].get(), grid_size), hGEOSCtxt);
+#endif
 						if (geom == nullptr)
 							Rcpp::stop("GEOS exception"); // #nocov
 						// ensure that geom is valid
@@ -1181,6 +1259,9 @@ Rcpp::List CPL_nary_intersection(Rcpp::List sfc) {
 	std::vector<GeomPtr> x = geometries_from_sfc(hGEOSCtxt, sfc, &dim);
 	std::vector<GeomPtr> out;
 	int errors = 0;
+#ifdef HAVE_390
+	double grid_size = geos_grid_size(sfc);
+#endif
 #ifdef HAVE350
 	notice = 0;
 	GEOSContext_setNoticeMessageHandler_r(hGEOSCtxt,
@@ -1209,7 +1290,11 @@ Rcpp::List CPL_nary_intersection(Rcpp::List sfc) {
 				// iterate over items in query and erase overlapping areas in geom
 				for (size_t j = 0; j < tree_sel.size(); j++) {
 					size_t k = tree_sel[j];
+#ifndef HAVE_390
 					GeomPtr inters = geos_ptr(GEOSIntersection_r(hGEOSCtxt, out[k].get(), geom.get()), hGEOSCtxt);
+#else
+					GeomPtr inters = geos_ptr(GEOSIntersectionPrec_r(hGEOSCtxt, out[k].get(), geom.get(), grid_size), hGEOSCtxt);
+#endif
 					if (geom.get() != nullptr) {
 						if (inters == nullptr)
 							errors++;
@@ -1219,7 +1304,11 @@ Rcpp::List CPL_nary_intersection(Rcpp::List sfc) {
 							if (geom == nullptr)
 								Rcpp::stop("GEOS exception"); // #nocov
 							// cut out inters from out[k]:
+#ifndef HAVE_390
 							GeomPtr g = geos_ptr(GEOSDifference_r(hGEOSCtxt, out[k].get(), inters.get()), hGEOSCtxt); 
+#else
+							GeomPtr g = geos_ptr(GEOSDifferencePrec_r(hGEOSCtxt, out[k].get(), inters.get(), grid_size), hGEOSCtxt); 
+#endif
 							if (g == nullptr)
 								Rcpp::warning("GEOS difference returns NULL"); // #nocov
 							else {
